@@ -1,5 +1,22 @@
 # coding: utf-8
 
+
+"""
+## Test contributions validity, and report suspicious contribution.
+
+The main function is `check_recent_changes(bot, days)`, and it will test contributions made in the <days> past days. 
+
+A test can inherits one of those two classes:
+
+* `BaseVersionTest()`:
+  * implements `test_document(document)`
+  * returns `True` if document is OK
+* `BaseContributionTest()`:
+  * implements `test_contribution(old_doc, new_doc)`
+  * returns `True` if going from old_doc to old_doc is OK (for instance, not a too big move)
+"""
+
+
 from __future__ import unicode_literals, print_function, division
 
 from dateutil import parser
@@ -27,7 +44,8 @@ def _get_diff_url(bot, document, lang, previous_version_id, version_id):
         document.document_id,
         lang,
         previous_version_id,
-        version_id)
+        version_id,
+    )
 
 
 class ContributionReport(object):
@@ -35,15 +53,19 @@ class ContributionReport(object):
         self.contrib = contrib
         self.need_report = False
 
-        self.new = bot.wiki.get_wiki_object_version(contrib.document.document_id,
-                                                    contrib.document.type,
-                                                    contrib.lang,
-                                                    contrib.version_id)
+        self.new = bot.wiki.get_wiki_object_version(
+            contrib.document.document_id,
+            contrib.document.type,
+            contrib.lang,
+            contrib.version_id,
+        )
 
-        self.old = bot.wiki.get_wiki_object_version(contrib.document.document_id,
-                                                    contrib.document.type,
-                                                    contrib.lang,
-                                                    self.new.previous_version_id)
+        self.old = bot.wiki.get_wiki_object_version(
+            contrib.document.document_id,
+            contrib.document.type,
+            contrib.lang,
+            self.new.previous_version_id,
+        )
 
         self.emojis = []
 
@@ -61,12 +83,14 @@ class ContributionReport(object):
 
     def get_multi_report(self, lang):
 
-        result = ("  * {timestamp} "
-                  "([{diff_title}]({diff_url})) **·** "
-                  "{delta} "
-                  "{emojis} **·** "
-                  "[{username}]({user_contrib_url}) →‎ "
-                  "*{comment}*").format(
+        result = (
+            "  * {timestamp} "
+            "([{diff_title}]({diff_url})) **·** "
+            "{delta} "
+            "{emojis} **·** "
+            "[{username}]({user_contrib_url}) →‎ "
+            "*{comment}*"
+        ).format(
             timestamp=parser.parse(self.contrib.written_at).strftime("%H:%M"),
             emojis="".join(self.emojis),
             diff_title="diff" if self.new.previous_version_id else "**new**",
@@ -74,22 +98,26 @@ class ContributionReport(object):
             delta=_format_delta(self.delta),
             username=self.contrib.user.name,
             user_contrib_url=self.contrib.user.get_contributions_url(),
-            comment=self.contrib.comment if len(self.contrib.comment) else "&nbsp;"
+            comment=self.contrib.comment if len(self.contrib.comment) else "&nbsp;",
         )
 
         return result
 
     def get_mono_report(self, bot, lang):
 
-        title = bot.wiki.get_wiki_object(self.new.document.document_id, self.new.document.type).get_title(lang)
+        title = bot.wiki.get_wiki_object(
+            self.new.document.document_id, self.new.document.type
+        ).get_title(lang)
 
-        result = ("* {timestamp} "
-                  "([{diff_title}]({diff_url}) | [hist]({hist_url})) **·** "
-                  "{delta} "
-                  "{emojis} **·** "
-                  "[{doc_title}]({doc_url}) **·** "
-                  "[{username}]({user_contrib_url}) →‎ "
-                  "*{comment}*").format(
+        result = (
+            "* {timestamp} "
+            "([{diff_title}]({diff_url}) | [hist]({hist_url})) **·** "
+            "{delta} "
+            "{emojis} **·** "
+            "[{doc_title}]({doc_url}) **·** "
+            "[{username}]({user_contrib_url}) →‎ "
+            "*{comment}*"
+        ).format(
             timestamp=parser.parse(self.contrib.written_at).strftime("%H:%M"),
             emojis="".join(self.emojis),
             doc_title=title if len(title) else "*Vide*",
@@ -100,7 +128,7 @@ class ContributionReport(object):
             delta=_format_delta(self.delta),
             username=self.contrib.user.name,
             user_contrib_url=self.contrib.user.get_contributions_url(),
-            comment=self.contrib.comment if len(self.contrib.comment) else "&nbsp;"
+            comment=self.contrib.comment if len(self.contrib.comment) else "&nbsp;",
         )
 
         return result
@@ -128,8 +156,9 @@ class DocumentReport(object):
 
             delta = sum([r.delta for r in self.sub_reports])
 
-            title = bot.wiki.get_wiki_object(newest_report.new.document.document_id,
-                                             newest_report.new.document.type).get_title(lang)
+            title = bot.wiki.get_wiki_object(
+                newest_report.new.document.document_id, newest_report.new.document.type
+            ).get_title(lang)
 
             result.append(
                 "* {timestamp} "
@@ -137,17 +166,26 @@ class DocumentReport(object):
                 "({delta}) **·** "
                 "[{doc_title}]({doc_url}) → "
                 "*{modifications} modifications*".format(
-                    timestamp=parser.parse(newest_report.contrib.written_at).strftime("%H:%M"),
+                    timestamp=parser.parse(newest_report.contrib.written_at).strftime(
+                        "%H:%M"
+                    ),
                     doc_title=title if len(title) else "*Vide*",
                     doc_url=newest_report.new.document.get_url(lang),
-                    hist_url=newest_report.new.document.get_history_url(newest_report.contrib.lang),
+                    hist_url=newest_report.new.document.get_history_url(
+                        newest_report.contrib.lang
+                    ),
                     diff_title="**new**" if not oldest_report.old else "diff",
-                    diff_url=_get_diff_url(bot, newest_report.contrib.document, lang,
-                                           oldest_report.new.previous_version_id,
-                                           newest_report.contrib.version_id),
+                    diff_url=_get_diff_url(
+                        bot,
+                        newest_report.contrib.document,
+                        lang,
+                        oldest_report.new.previous_version_id,
+                        newest_report.contrib.version_id,
+                    ),
                     modifications=len(self.sub_reports),
-                    delta=_format_delta(delta)
-                ))
+                    delta=_format_delta(delta),
+                )
+            )
 
             for report in self.sub_reports:
                 result.append(report.get_multi_report(lang))
@@ -156,7 +194,9 @@ class DocumentReport(object):
 
 
 def check_recent_changes(bot, days, ask_before_saving):
-    check_message_url = "https://forum.camptocamp.org/t/topoguide-verifications-automatiques/201480"
+    check_message_url = (
+        "https://forum.camptocamp.org/t/topoguide-verifications-automatiques/201480"
+    )
     lang = "fr"
 
     newest_date = utils.today().replace(hour=0, minute=0, second=0, microsecond=0)
@@ -167,7 +207,9 @@ def check_recent_changes(bot, days, ask_before_saving):
     tests = get_fixed_tests(lang)
     tests += get_re_tests(bot.forum.get_post(url=check_message_url), lang)
 
-    items = bot.get_modified_documents(lang=lang, oldest_date=oldest_date, newest_date=newest_date).values()
+    items = bot.get_modified_documents(
+        lang=lang, oldest_date=oldest_date, newest_date=newest_date
+    ).values()
 
     reports = []
 
@@ -189,7 +231,9 @@ def check_recent_changes(bot, days, ask_before_saving):
         messages.append("</tr>")
 
     messages.append("</table>\n[/details]\n\n----\n\n")
-    messages += [report.get_report(bot, lang) for report in reports if report.need_report]
+    messages += [
+        report.get_report(bot, lang) for report in reports if report.need_report
+    ]
 
     for m in messages:
         print(m)
@@ -226,28 +270,74 @@ def get_re_tests(configuration, lang):
 
 
 def get_fixed_tests(lang):
-    return [HistoryTest(lang), LengthTest(lang),
-            NewbieTest(), MainWaypointTest(),
-            RouteTypeTest(), DistanceTest()]
+    return [
+        HistoryTest(lang),
+        LengthTest(lang),
+        NewbieTest(),
+        MainWaypointTest(),
+        RouteTypeTest(),
+        DistanceTest(),
+        QualityTest(),
+    ]
 
 
-class LengthTest(object):
-    def __init__(self, lang):
-        self.name = "Grosse suppression"
+class _BaseTest(object):
+    def __init__(self, name, lang=None):
+        self.name = name
         self.lang = lang
 
+    def __call__(self, contrib, old_version, new_version):
+        raise NotImplementedError()
+
+
+class BaseVersionTest(_BaseTest):
+    def __init__(self, name, lang=None):
+        super(BaseVersionTest, self).__init__(name, lang)
+
+    def __call__(self, contrib, old_version, new_version):
+        return self.test_version(old_version), self.test_version(new_version)
+
+    def test_version(self, version):
+        if not version or not version.document:
+            return True
+
+        if "redirects_to" in version.document:
+            return True
+
+        return self.test_document(document=version.document)
+
+    def test_document(self, document):
+        raise NotImplementedError()
+
+
+class BaseContributionTest(_BaseTest):
+    def __init__(self, name, lang=None):
+        super(BaseContributionTest, self).__init__(name, lang)
+
+    def __call__(self, contrib, old_version, new_version):
+        if not old_version or not new_version:
+            return True, True
+
+        if not old_version.document or not new_version.document:
+            return True, True
+
+        old_doc = old_version.document
+        new_doc = new_version.document
+
+        if "redirects_to" in old_doc or "redirects_to" in new_doc:
+            return True, True
+
+        return True, self.test_contribution(old_doc, new_doc)
+
+
+class LengthTest(BaseContributionTest):
+    def __init__(self, lang):
+        super(LengthTest, self).__init__(name="Grosse suppression", lang=lang)
+
         self.fail_marker = emoji("/images/emoji/apple/rage.png?v=3", self.name)
         self.success_marker = ""
 
-    def __call__(self, contrib, old_version, new_version):
-        old_doc = old_version.document if old_version else None
-        new_doc = new_version.document if new_version else None
-
-        if not old_doc or "redirects_to" in old_doc:
-            return True, True
-
-        if not new_doc or "redirects_to" in new_doc:
-            return True, True
+    def test_contribution(self, old_doc, new_doc):
 
         result = True
 
@@ -257,12 +347,12 @@ class LengthTest(object):
         if old_locale_length != 0 and new_locale_length / old_locale_length < 0.5:
             result = False
 
-        return True, result
+        return result
 
 
-class NewbieTest(object):
+class NewbieTest(_BaseTest):
     def __init__(self):
-        self.name = "Nouvel utilisateur"
+        super(NewbieTest, self).__init__(name="Nouvel utilisateur")
 
         self.fail_marker = emoji("/images/emoji/apple/gift.png?v=3", self.name)
         self.success_marker = ""
@@ -271,122 +361,120 @@ class NewbieTest(object):
         if contrib.user.is_newbie():
             return True, False
         else:
-            return False, False
+            return True, True
 
 
-class ReTest(object):
+class ReTest(BaseVersionTest):
     def __init__(self, name, lang):
-        self.name = name
-        self.lang = lang
+        super(ReTest, self).__init__(name=name, lang=lang)
         self.patterns = []
         self.fail_marker = emoji("/images/emoji/apple/red_circle.png?v=3", self.name)
-        self.success_marker = emoji("/images/emoji/apple/white_check_mark.png?v=3",
-                                    self.name + " corrigé")
+        self.success_marker = emoji(
+            "/images/emoji/apple/white_check_mark.png?v=3", self.name + " corrigé"
+        )
 
-    def __call__(self, contrib, old_version, new_version):
-        old_doc = old_version.document if old_version else None
-        new_doc = new_version.document if new_version else None
-
-        def test(doc):
-            if not doc or "redirects_to" in doc:
-                return True
-
-            return not doc.search(self.patterns, self.lang)
-
-        return test(old_doc), test(new_doc)
+    def test_document(self, document):
+        return not document.search(self.patterns, self.lang)
 
 
-class HistoryTest(object):
-    activities_with_history = ["snow_ice_mixed", "mountain_climbing", "rock_climbing", "ice_climbing"]
+class HistoryTest(BaseVersionTest):
+    activities_with_history = [
+        "snow_ice_mixed",
+        "mountain_climbing",
+        "rock_climbing",
+        "ice_climbing",
+    ]
 
     def __init__(self, lang):
-        self.name = "Champ historique"
-        self.lang = lang
+        super(HistoryTest, self).__init__(name="Champ historique", lang=lang)
         self.fail_marker = emoji("/images/emoji/apple/closed_book.png?v=3", self.name)
-        self.success_marker = emoji("/images/emoji/apple/green_book.png?v=3", self.name + " rempli")
-
-    def __call__(self, contrib, old_version, new_version):
-        old_doc = old_version.document if old_version else None
-        new_doc = new_version.document if new_version else None
-
-        def test(doc):
-            if not doc or "redirects_to" in doc or doc.type != "r":
-                return True
-
-            if len([act for act in doc.activities if act in self.activities_with_history]) == 0:
-                return True
-
-            locale = doc.get_locale(self.lang)
-            if locale and (not locale.route_history or len(locale.route_history) == 0):
-                return False
+        self.success_marker = emoji(
+            "/images/emoji/apple/green_book.png?v=3", self.name + " rempli"
+        )
 
+    def test_document(self, document):
+        if document.type != "r":
             return True
 
-        return test(old_doc), test(new_doc)
+        if (
+            len(
+                [
+                    act
+                    for act in document.activities
+                    if act in self.activities_with_history
+                ]
+            )
+            == 0
+        ):
+            return True
+
+        locale = document.get_locale(self.lang)
+        if locale and (not locale.route_history or len(locale.route_history) == 0):
+            return False
+
+        return True
 
 
-class MainWaypointTest(object):
+class MainWaypointTest(BaseVersionTest):
     def __init__(self):
-        self.name = "Main waypoint"
-        self.fail_marker = emoji("https://forum.camptocamp.org/uploads/default/original/2X/f/f2c72706b83fd5bd21e110cb1b9758c763905023.png", self.name)
-        self.success_marker = emoji("https://forum.camptocamp.org/uploads/default/original/2X/3/37abfd096a21bed932bea1d7150b9264abc12476.png", self.name + " corrigé")
+        super(MainWaypointTest, self).__init__(name="Main waypoint")
 
-    def __call__(self, contrib, old_version, new_version):
-        if new_version.document.type != "r":
-            return True, True
+        self.fail_marker = emoji(
+            "https://forum.camptocamp.org/uploads/default/original/2X/f/f2c72706b83fd5bd21e110cb1b9758c763905023.png",
+            self.name,
+        )
+        self.success_marker = emoji(
+            "https://forum.camptocamp.org/uploads/default/original/2X/3/37abfd096a21bed932bea1d7150b9264abc12476.png",
+            self.name + " corrigé",
+        )
 
-        def test(version):
-            if not version:
-                return True
+    def test_document(self, document):
+        if document.type != "r":
+            return True
 
-            if "redirects_to" in version.document:
-                return True
-
-            return version.document.main_waypoint_id is not None
-
-        return test(old_version), test(new_version)
+        return document.main_waypoint_id is not None
 
 
-class RouteTypeTest(object):
+class RouteTypeTest(BaseVersionTest):
     def __init__(self):
-        self.name = "Type de voie renseigné"
+        super(RouteTypeTest, self).__init__(name="Type de voie renseigné")
         self.fail_marker = emoji("/images/emoji/apple/red_circle.png?v=3", self.name)
-        self.success_marker = emoji("/images/emoji/apple/white_check_mark.png?v=3",
-                                    self.name + " corrigé")
+        self.success_marker = emoji(
+            "/images/emoji/apple/white_check_mark.png?v=3", self.name + " corrigé"
+        )
 
-    def __call__(self, contrib, old_version, new_version):
-        def test(version):
-            if not version:
-                return True
+    def test_document(self, document):
 
-            if "redirects_to" in version.document:
-                return True
+        if document.type != "r" or "rock_climbing" not in document.activities:
+            return True
 
-            if version.document.type != "r" or "rock_climbing" not in version.document.activities:
-                return True
-
-            climbing_outdoor_type = version.document.climbing_outdoor_type
-            return climbing_outdoor_type is not None and len(climbing_outdoor_type) != 0
-
-        return test(old_version), test(new_version)
+        climbing_outdoor_type = document.climbing_outdoor_type
+        return climbing_outdoor_type is not None and len(climbing_outdoor_type) != 0
 
 
-class DistanceTest(object):
+class DistanceTest(BaseContributionTest):
     def __init__(self):
-        self.name = "Gros déplacement géographique"
-        self.fail_marker = emoji("/uploads/default/original/2X/0/0178043b1b70e669946f609571bd4b8f7d18e820.png",
-                                 self.name)
+        super(DistanceTest, self).__init__(name="Gros déplacement géographique")
+        self.fail_marker = emoji(
+            "/uploads/default/original/2X/0/0178043b1b70e669946f609571bd4b8f7d18e820.png",
+            self.name,
+        )
         self.success_marker = ""
 
-    def __call__(self, contrib, old_version, new_version):
-        if old_version is None or new_version is None:
-            return True, True
-
-        old_doc = old_version.document
-        new_doc = new_version.document
-
-        if "redirects_to" in old_doc or "redirects_to" in new_doc:
-            return True, True
-
+    def test_contribution(self, old_doc, new_doc):
         distance = utils.compute_distance(old_doc, new_doc)
-        return True, distance is None or distance < 10
+        return distance is None or distance < 500
+
+
+class QualityTest(BaseContributionTest):
+    """
+    Report when quality field changes
+    """
+
+    def __init__(self):
+        super(QualityTest, self).__init__(name="Changement du champ qualité")
+        self.fail_marker = "🧹"
+        self.success_marker = ""
+
+    def test_contribution(self, old_doc, new_doc):
+        return old_doc.quality == new_doc.quality
